
from django.forms import CharField, FloatField, FileField, BooleanField, IntegerField, ChoiceField
from form_utils.forms import BetterForm


field_map = {
    'char': CharField,
    'float': FloatField,
    'file': FileField,
    'bool': BooleanField,
    'int': IntegerField,
    'choice': ChoiceField,
}


def generate_form(parsed):
    """
    Generate a django form from a parsed kliko object
    """
    all_fields = {}
    fieldsets = []
    for section in parsed['sections']:
        fields_in_section = []

        for field in section['fields']:
            kwargs = {}
            for kwarg in ('initial', 'max_length', 'label', 'help_text', 'file'):
                if kwarg in field:
                    kwargs[kwarg] = field[kwarg]
            if 'choices' in field:
                kwargs['choices'] = field['choices'].items()
            fields_in_section.append(field['name'])
            all_fields[field['name']] = field_map[field['type']](**kwargs)

        fieldsets.append((section['name'],
                          {'fields': fields_in_section,
                           'description': section['description']}
                          ))

    Meta = type('Meta', (), {'fieldsets': fieldsets})
    all_fields['Meta'] = Meta

    return type('Form', (BetterForm,), all_fields)



import os
import unittest

import kliko.validate

here = os.path.dirname(os.path.realpath(__file__))


class TestExample(unittest.TestCase):
    def test_parse(self):
        example = open(os.path.join(here, "../examples/form.yml"), 'r').read()
        kliko.validate.validate(example)






from pykwalify.core import Core
import yaml
import os


here = os.path.dirname(os.path.realpath(__file__))

schema_file = os.path.join(here, "schema.yml")
schema_str = open(schema_file, 'r').read()
schema = yaml.load(schema_str)


def validate(str_):
    # first try to parse it, to make sure it is parsable
    parsed = yaml.load(str_)

    c = Core(source_data=parsed, schema_files=[schema_file])
    c.validate(raise_exception=True)
    return yaml.load(str_)



import logging
from .exceptions import KlikoException


logger = logging.getLogger(__name__)

def extract_params(docker_client, image_name):
    """
    args:
        docker_client (docker.docker.Client): a docker client object
        image_name (str): name of the image to use for param_spec.yml extraction
    returns:
        str: content of the param schema
    """
    container = docker_client.create_container(image=image_name, command=['/bin/cat', '/param_spec.yml'])
    docker_client.start(container)
    error_code = docker_client.wait(container)
    warnings = container.get('Warnings')
    if warnings:
        for warning in warnings:
            logger.warning(warning)
    stdout = docker_client.logs(container, stdout=True, stream=False).decode('utf-8')
    if error_code != 0:
        raise KlikoException(stdout.strip())
    return stdout



import os
import unittest
import docker
import docker.utils
from builtins import open

import kliko.utils
from kliko.exceptions import KlikoException

TEST_IMAGE = 'klikotest'

TEST_ARCHIVE = os.path.join(os.path.dirname(__file__), 'test_image/%s.tar' % TEST_IMAGE)
PARAMS_FILE = os.path.join(os.path.dirname(__file__), 'test_image/param_spec.yml')


def set_fixture(docker_client, image_name, image_archive):
        if docker_client.images(name=image_name):
            docker_client.remove_image(image=image_name, force=True)
        docker_client.import_image_from_file(filename=image_archive, repository=image_name, tag='latest')


class TestUtils(unittest.TestCase):
    def setUp(self):
        config = docker.utils.kwargs_from_env()
        config['version'] = "1.20"
        self.client = docker.Client(**config)
        set_fixture(self.client, TEST_IMAGE, TEST_ARCHIVE)

    def test_extract(self):
        image_params = kliko.utils.extract_params(self.client, TEST_IMAGE)
        true_params = open(PARAMS_FILE, mode='r', encoding='utf-8').read()
        self.assertEqual(image_params, true_params)

    def test_extract_without_params(self):
        with self.assertRaises(KlikoException):
            kliko.utils.extract_params(self.client, 'alpine:3.3')



from setuptools import setup, find_packages

from kliko.version import __version__

setup(
    name="kliko",
    version=__version__,
    packages=find_packages(),
    scripts=[],
    install_requires=['pykwalify', 'PyYAML', 'docker-py', 'future'],

    package_data={
        '': ['*.txt', '*.rst'],
        'kliko': ['*.yml'],
    },

    author="Gijs Molenaar",
    author_email="gijs@pythonic.nl",
    description="Scientific Compute Container Spec Parser",
    license="GPL2",
    keywords="science docker yaml json",
    url="https://github.com/gijzelaerr/kliko",
)
